@@ -133,8 +133,6 @@ static gboolean gst_initialized = FALSE;
 static GList *plugin_paths = NULL;      /* for delayed processing in post_init */
 #endif
 
-extern gint _gst_trace_on;
-
 #ifndef GST_DISABLE_GST_DEBUG
 extern const gchar *priv_gst_dump_dot_dir;
 #endif
@@ -627,21 +625,6 @@ gst_register_core_elements (GstPlugin * plugin)
   return TRUE;
 }
 
-static GstPluginDesc plugin_desc = {
-  GST_VERSION_MAJOR,
-  GST_VERSION_MINOR,
-  "staticelements",
-  "core elements linked into the GStreamer library",
-  gst_register_core_elements,
-  VERSION,
-  GST_LICENSE,
-  PACKAGE,
-  GST_PACKAGE_NAME,
-  GST_PACKAGE_ORIGIN,
-
-  GST_PADDING_INIT
-};
-
 #ifndef GST_DISABLE_REGISTRY
 
 typedef enum
@@ -993,11 +976,14 @@ init_post (GOptionContext * context, GOptionGroup * group, gpointer data,
   _gst_message_initialize ();
   _gst_tag_initialize ();
 
-  /* register core plugins */
-  _gst_plugin_register_static (&plugin_desc);
-
   _gst_plugin_initialize ();
 
+  /* register core plugins */
+  gst_plugin_register_static (GST_VERSION_MAJOR, GST_VERSION_MINOR,
+      "staticelements", "core elements linked into the GStreamer library",
+      gst_register_core_elements, VERSION, GST_LICENSE, PACKAGE,
+      GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN);
+
   /*
    * Any errors happening below this point are non-fatal, we therefore mark
    * gstreamer as being initialized, since it is the case from a plugin point of
