@@ -217,7 +217,7 @@ gst_object_class_init (GstObjectClass * klass)
   gst_object_signals[PARENT_SET] =
       g_signal_new ("parent-set", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
       G_STRUCT_OFFSET (GstObjectClass, parent_set), NULL, NULL,
-      g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, G_TYPE_OBJECT);
+      g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, GST_TYPE_OBJECT);
 
   /**
    * GstObject::parent-unset:
@@ -229,7 +229,7 @@ gst_object_class_init (GstObjectClass * klass)
   gst_object_signals[PARENT_UNSET] =
       g_signal_new ("parent-unset", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstObjectClass, parent_unset), NULL,
-      NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, G_TYPE_OBJECT);
+      NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, GST_TYPE_OBJECT);
 
 #ifndef GST_DISABLE_LOADSAVE
   /**
@@ -265,7 +265,7 @@ gst_object_class_init (GstObjectClass * klass)
       g_signal_new ("deep-notify", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_FIRST | G_SIGNAL_NO_RECURSE | G_SIGNAL_DETAILED |
       G_SIGNAL_NO_HOOKS, G_STRUCT_OFFSET (GstObjectClass, deep_notify), NULL,
-      NULL, gst_marshal_VOID__OBJECT_PARAM, G_TYPE_NONE, 2, G_TYPE_OBJECT,
+      NULL, gst_marshal_VOID__OBJECT_PARAM, G_TYPE_NONE, 2, GST_TYPE_OBJECT,
       G_TYPE_PARAM);
 
   klass->path_string_separator = "/";
@@ -304,7 +304,7 @@ gst_object_init (GTypeInstance * instance, gpointer g_class)
  * gst_object_ref:
  * @object: a #GstObject to reference
  *
- * Increments the refence count on @object. This function
+ * Increments the reference count on @object. This function
  * does not take the lock on @object because it relies on
  * atomic refcounting.
  *
@@ -333,7 +333,7 @@ gst_object_ref (gpointer object)
  * gst_object_unref:
  * @object: a #GstObject to unreference
  *
- * Decrements the refence count on @object.  If reference count hits
+ * Decrements the reference count on @object.  If reference count hits
  * zero, destroy @object. This function does not take the lock
  * on @object as it relies on atomic refcounting.
  *
@@ -517,23 +517,14 @@ gst_object_dispatch_properties_changed (GObject * object,
   /* now let the parent dispatch those, too */
   parent = gst_object_get_parent (gst_object);
   while (parent) {
-    /* for debugging ... */
-    gchar *parent_name = gst_object_get_name (parent);
-
-#ifndef GST_DISABLE_GST_DEBUG
-    gchar *debug_parent_name = GST_STR_NULL (parent_name);
-#endif
-
-    /* need own category? */
     for (i = 0; i < n_pspecs; i++) {
-      GST_CAT_LOG (GST_CAT_EVENT, "deep notification from %s to %s (%s)",
-          debug_name, debug_parent_name, pspecs[i]->name);
+      GST_LOG_OBJECT (parent, "deep notification from %s (%s)",
+          debug_name, pspecs[i]->name);
 
       g_signal_emit (parent, gst_object_signals[DEEP_NOTIFY],
           g_quark_from_string (pspecs[i]->name), GST_OBJECT_CAST (object),
           pspecs[i]);
     }
-    g_free (parent_name);
 
     old_parent = parent;
     parent = gst_object_get_parent (old_parent);
