@@ -28,7 +28,7 @@
  *
  * One registry holds the metadata of a set of plugins.
  * All registries build the #GstRegistryPool.
- * 
+ *
  * <emphasis role="bold">Design:</emphasis>
  *
  * The #GstRegistry object is a list of plugins and some functions for dealing
@@ -109,7 +109,7 @@
 
 #define GST_CAT_DEFAULT GST_CAT_REGISTRY
 
-/* the one instance of the default registry and the mutex protecting the 
+/* the one instance of the default registry and the mutex protecting the
  * variable. */
 static GStaticMutex _gst_registry_mutex = G_STATIC_MUTEX_INIT;
 static GstRegistry *_gst_registry_default = NULL;
@@ -122,8 +122,6 @@ enum
   LAST_SIGNAL
 };
 
-static void gst_registry_class_init (GstRegistryClass * klass);
-static void gst_registry_init (GstRegistry * registry);
 static void gst_registry_finalize (GObject * object);
 
 static guint gst_registry_signals[LAST_SIGNAL] = { 0 };
@@ -177,6 +175,7 @@ gst_registry_class_init (GstRegistryClass * klass)
 static void
 gst_registry_init (GstRegistry * registry)
 {
+  registry->feature_hash = g_hash_table_new (g_str_hash, g_str_equal);
 }
 
 static void
@@ -219,6 +218,8 @@ gst_registry_finalize (GObject * object)
   }
   g_list_free (features);
 
+  g_hash_table_destroy (registry->feature_hash);
+  registry->feature_hash = NULL;
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -384,6 +385,7 @@ gst_registry_remove_features_for_plugin_unlocked (GstRegistry * registry,
           gst_plugin_get_name (plugin));
 
       registry->features = g_list_delete_link (registry->features, f);
+      g_hash_table_remove (registry->feature_hash, feature->name);
       gst_object_unref (feature);
     }
     f = next;
@@ -441,15 +443,22 @@ gst_registry_add_feature (GstRegistry * registry, GstPluginFeature * feature)
   existing_feature = gst_registry_lookup_feature_locked (registry,
       feature->name);
   if (G_UNLIKELY (existing_feature)) {
-    GST_DEBUG_OBJECT (registry, "Replacing existing feature %p (%s)",
+    GST_DEBUG_OBJECT (registry, "replacing existing feature %p (%s)",
         existing_feature, feature->name);
+    /* Remove the existing feature from the list now, before we insert the new
+     * one, but don't unref yet because the hash is still storing a reference to     * it. */
     registry->features = g_list_remove (registry->features, existing_feature);
-    gst_object_unref (existing_feature);
   }
 
   GST_DEBUG_OBJECT (registry, "adding feature %p (%s)", feature, feature->name);
 
   registry->features = g_list_prepend (registry->features, feature);
+  g_hash_table_replace (registry->feature_hash, feature->name, feature);
+
+  if (G_UNLIKELY (existing_feature)) {
+    /* We unref now. No need to remove the feature name from the hash table, it      * got replaced by the new feature */
+    gst_object_unref (existing_feature);
+  }
 
   gst_object_ref (feature);
   gst_object_sink (feature);
@@ -482,6 +491,7 @@ gst_registry_remove_feature (GstRegistry * registry, GstPluginFeature * feature)
 
   GST_OBJECT_LOCK (registry);
   registry->features = g_list_remove (registry->features, feature);
+  g_hash_table_remove (registry->feature_hash, feature->name);
   GST_OBJECT_UNLOCK (registry);
   gst_object_unref (feature);
 }
@@ -695,21 +705,10 @@ gst_registry_get_plugin_list (GstRegistry * registry)
 static GstPluginFeature *
 gst_registry_lookup_feature_locked (GstRegistry * registry, const char *name)
 {
-  GList *g;
-  GstPluginFeature *feature;
-
   if (G_UNLIKELY (name == NULL))
     return NULL;
 
-  /* FIXME: use GTree speed up lookups */
-  for (g = registry->features; g; g = g_list_next (g)) {
-    feature = GST_PLUGIN_FEATURE_CAST (g->data);
-    if (feature->name && strcmp (name, feature->name) == 0) {
-      return feature;
-    }
-  }
-
-  return NULL;
+  return g_hash_table_lookup (registry->feature_hash, name);
 }
 
 /**
@@ -718,7 +717,7 @@ gst_registry_lookup_feature_locked (GstRegistry * registry, const char *name)
  * @name: a #GstPluginFeature name
  *
  * Find a #GstPluginFeature with @name in @registry.
- * 
+ *
  * Returns: a #GstPluginFeature with its refcount incremented, use
  * gst_object_unref() after usage.
  *
@@ -867,7 +866,7 @@ gst_registry_scan_path_level (GstRegistry * registry, const gchar * path,
         GST_LOG_OBJECT (registry, "marking plugin %p as registered as %s",
             plugin, filename);
         plugin->registered = TRUE;
-        /* Update the file path on which we've seen this cached plugin 
+        /* Update the file path on which we've seen this cached plugin
          * to ensure the registry cache will reflect up to date information */
         if (strcmp (plugin->filename, filename) != 0) {
           g_free (plugin->filename);
@@ -881,6 +880,9 @@ gst_registry_scan_path_level (GstRegistry * registry, const gchar * path,
             G_GINT64_FORMAT, plugin->file_mtime, file_status.st_mtime,
             (gint64) plugin->file_size, (gint64) file_status.st_size);
         gst_registry_remove_plugin (gst_registry_get_default (), plugin);
+        /* We don't use a GError here because a failure to load some shared 
+         * objects as plugins is normal (particularly in the uninstalled case)
+         */
         newplugin = gst_plugin_load_file (filename, NULL);
         if (newplugin) {
           GST_DEBUG_OBJECT (registry, "marking new plugin %p as registered",
