@@ -274,15 +274,13 @@ gst_segment_set_seek (GstSegment * segment, gdouble rate,
 
   if (G_UNLIKELY (segment->format == GST_FORMAT_UNDEFINED))
     segment->format = format;
-  else
-    g_return_if_fail (segment->format == format);
 
   update_start = update_stop = TRUE;
 
-  /* start is never invalid */
+  /* segment->start is never invalid */
   switch (start_type) {
     case GST_SEEK_TYPE_NONE:
-      /* no update to segment */
+      /* no update to segment, take previous start */
       start = segment->start;
       update_start = FALSE;
       break;
@@ -290,13 +288,17 @@ gst_segment_set_seek (GstSegment * segment, gdouble rate,
       /* start holds desired position, map -1 to the start */
       if (start == -1)
         start = 0;
+      /* start must be 0 or the formats must match */
+      g_return_if_fail (start == 0 || segment->format == format);
       break;
     case GST_SEEK_TYPE_CUR:
-      /* add start to currently configure segment */
+      g_return_if_fail (start == 0 || segment->format == format);
+      /* add start to currently configured segment */
       start = segment->start + start;
       break;
     case GST_SEEK_TYPE_END:
       if (segment->duration != -1) {
+        g_return_if_fail (start == 0 || segment->format == format);
         /* add start to total length */
         start = segment->duration + start;
       } else {
@@ -319,18 +321,24 @@ gst_segment_set_seek (GstSegment * segment, gdouble rate,
       update_stop = FALSE;
       break;
     case GST_SEEK_TYPE_SET:
-      /* stop holds required value */
+      /* stop holds required value, if it's not -1, it must be of the same
+       * format as the segment. */
+      g_return_if_fail (stop == -1 || segment->format == format);
       break;
     case GST_SEEK_TYPE_CUR:
-      if (segment->stop != -1)
+      if (segment->stop != -1) {
+        /* only add compatible formats or 0 */
+        g_return_if_fail (stop == 0 || segment->format == format);
         stop = segment->stop + stop;
-      else
+      } else
         stop = -1;
       break;
     case GST_SEEK_TYPE_END:
-      if (segment->duration != -1)
+      if (segment->duration != -1) {
+        /* only add compatible formats or 0 */
+        g_return_if_fail (stop == 0 || segment->format == format);
         stop = segment->duration + stop;
-      else {
+      } else {
         stop = segment->stop;
         update_stop = FALSE;
       }
@@ -373,8 +381,7 @@ gst_segment_set_seek (GstSegment * segment, gdouble rate,
     *update = last_stop != segment->last_stop;
 
   /* update new position */
-  if (last_stop != segment->last_stop)
-    segment->last_stop = last_stop;
+  segment->last_stop = last_stop;
 
   segment->time = start;
   segment->stop = stop;
@@ -443,12 +450,21 @@ gst_segment_set_newsegment_full (GstSegment * segment, gboolean update,
   g_return_if_fail (segment->format == format);
 
   if (update) {
-    /* an update to the current segment is done, elapsed time is
-     * difference between the old start and new start. */
-    if (start > segment->start)
-      duration = start - segment->start;
-    else
-      duration = 0;
+    if (segment->rate > 0.0) {
+      /* an update to the current segment is done, elapsed time is
+       * difference between the old start and new start. */
+      if (start > segment->start)
+        duration = start - segment->start;
+      else
+        duration = 0;
+    } else {
+      /* for negative rates, the elapsed duration is the diff between the stop
+       * positions */
+      if (stop != -1 && stop < segment->stop)
+        duration = segment->stop - stop;
+      else
+        duration = 0;
+    }
   } else {
     /* the new segment has to be aligned with the old segment.
      * We first update the accumulated time of the previous
@@ -508,7 +524,7 @@ gint64
 gst_segment_to_stream_time (GstSegment * segment, GstFormat format,
     gint64 position)
 {
-  gint64 result;
+  gint64 result, start, stop, time;
   gdouble abs_applied_rate;
 
   g_return_val_if_fail (segment != NULL, -1);
@@ -519,23 +535,33 @@ gst_segment_to_stream_time (GstSegment * segment, GstFormat format,
 
   if (G_UNLIKELY (segment->format == GST_FORMAT_UNDEFINED))
     segment->format = format;
-  else
-    g_return_val_if_fail (segment->format == format, -1);
+
+  /* if we have the position for the same format as the segment, we can compare
+   * the start and stop values, otherwise we assume 0 and -1 */
+  if (segment->format == format) {
+    start = segment->start;
+    stop = segment->stop;
+    time = segment->time;
+  } else {
+    start = 0;
+    stop = -1;
+    time = 0;
+  }
 
   /* outside of the segment boundary stop */
-  if (G_UNLIKELY (segment->stop != -1 && position > segment->stop))
+  if (G_UNLIKELY (stop != -1 && position > stop))
     return -1;
 
   /* before the segment boundary */
-  if (G_UNLIKELY (position < segment->start))
+  if (G_UNLIKELY (position < start))
     return -1;
 
   /* time must be known */
-  if (G_UNLIKELY (segment->time == -1))
+  if (G_UNLIKELY (time == -1))
     return -1;
 
   /* bring to uncorrected position in segment */
-  result = position - segment->start;
+  result = position - start;
 
   abs_applied_rate = ABS (segment->applied_rate);
 
@@ -546,11 +572,11 @@ gst_segment_to_stream_time (GstSegment * segment, GstFormat format,
   /* add or subtract from segment time based on applied rate */
   if (segment->applied_rate > 0.0) {
     /* correct for segment time */
-    result += segment->time;
+    result += time;
   } else {
     /* correct for segment time, clamp at 0 */
-    if (segment->time > result)
-      result = segment->time - result;
+    if (time > result)
+      result = time - result;
     else
       result = 0;
   }
@@ -583,6 +609,7 @@ gst_segment_to_running_time (GstSegment * segment, GstFormat format,
     gint64 position)
 {
   gint64 result;
+  gint64 start, stop, accum;
 
   g_return_val_if_fail (segment != NULL, -1);
 
@@ -591,28 +618,38 @@ gst_segment_to_running_time (GstSegment * segment, GstFormat format,
 
   if (G_UNLIKELY (segment->format == GST_FORMAT_UNDEFINED))
     segment->format = format;
-  else if (segment->accum)
-    g_return_val_if_fail (segment->format == format, -1);
+
+  /* if we have the position for the same format as the segment, we can compare
+   * the start and stop values, otherwise we assume 0 and -1 */
+  if (segment->format == format) {
+    start = segment->start;
+    stop = segment->stop;
+    accum = segment->accum;
+  } else {
+    start = 0;
+    stop = -1;
+    accum = 0;
+  }
 
   /* before the segment boundary */
-  if (G_UNLIKELY (position < segment->start))
+  if (G_UNLIKELY (position < start))
     return -1;
 
   if (segment->rate > 0.0) {
     /* outside of the segment boundary stop */
-    if (G_UNLIKELY (segment->stop != -1 && position > segment->stop))
+    if (G_UNLIKELY (stop != -1 && position > stop))
       return -1;
 
     /* bring to uncorrected position in segment */
-    result = position - segment->start;
+    result = position - start;
   } else {
     /* cannot continue if no stop position set or outside of
      * the segment. */
-    if (G_UNLIKELY (segment->stop == -1 || position > segment->stop))
+    if (G_UNLIKELY (stop == -1 || position > stop))
       return -1;
 
     /* bring to uncorrected position in segment */
-    result = segment->stop - position;
+    result = stop - position;
   }
 
   /* scale based on the rate, avoid division by and conversion to 
@@ -621,7 +658,7 @@ gst_segment_to_running_time (GstSegment * segment, GstFormat format,
     result /= segment->abs_rate;
 
   /* correct for accumulated segments */
-  result += segment->accum;
+  result += accum;
 
   return result;
 }
@@ -646,6 +683,9 @@ gst_segment_to_running_time (GstSegment * segment, GstFormat format,
  * updated. If @clip_start or @clip_stop are different from @start or @stop
  * respectively, the region fell partially in the segment.
  *
+ * Note that when @stop is -1, @clip_stop will be set to the end of the
+ * segment. Depending on the use case, this may or may not be what you want.
+ *
  * Returns: TRUE if the given @start and @stop times fall partially or 
  *     completely in @segment, FALSE if the values are completely outside 
  *     of the segment.
