@@ -170,7 +170,8 @@ struct _GstBaseTransform {
  * @transform_ip:   Required if the element operates in-place.
  *                  Transform the incoming buffer in-place.
  * @event:          Optional.
- *                  Event handler on the sink pad.
+ *                  Event handler on the sink pad. This function should return
+ *                  TRUE if the base class should forward the event. 
  * @src_event:      Optional.
  *                  Event handler on the source pad.
  * @passthrough_on_same_caps: If set to TRUE, passthrough mode will be
@@ -252,6 +253,9 @@ void		gst_base_transform_set_qos_enabled  (GstBaseTransform *trans,
 		                                     gboolean enabled);
 gboolean	gst_base_transform_is_qos_enabled   (GstBaseTransform *trans);
 
+void            gst_base_transform_set_gap_aware    (GstBaseTransform *trans,
+                                                     gboolean gap_aware);
+
 G_END_DECLS
 
 #endif /* __GST_BASE_TRANSFORM_H__ */
