@@ -220,10 +220,14 @@ get_candidates (const gchar * dir, const gchar * base)
   g_free (pattern);
 
   /* get all dirs from the path and prepend with given dir */
-  path = g_strdup_printf ("%s%c%s",
-      dir, G_SEARCHPATH_SEPARATOR, g_getenv ("PATH"));
+  if (dir)
+    path = g_strdup_printf ("%s%c%s",
+        dir, G_SEARCHPATH_SEPARATOR, g_getenv ("PATH"));
+  else
+    path = (gchar *) g_getenv ("PATH");
   dirs = g_strsplit (path, G_SEARCHPATH_SEPARATOR_S, 0);
-  g_free (path);
+  if (dir)
+    g_free (path);
 
   /* check all of these in reverse order by winding to bottom and going up  */
   cur = &dirs[0];
@@ -232,8 +236,10 @@ get_candidates (const gchar * dir, const gchar * base)
 
   while (cur != &dirs[0]) {
     --cur;
-    if (!g_file_test (*cur, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR))
+    if (!g_file_test (*cur, G_FILE_TEST_EXISTS) ||
+        !g_file_test (*cur, G_FILE_TEST_IS_DIR)) {
       continue;
+    }
 
     gdir = g_dir_open (*cur, 0, &error);
     if (!gdir) {
@@ -289,12 +295,18 @@ main (int argc, char **argv)
   gchar *highest = NULL;
   gchar *binary;                /* actual binary we're going to run */
   gchar *path = NULL;           /* and its path */
+  gchar *desc;
   GOptionContext *ctx;
   GError *err = NULL;
-  int nextopt;
+
+  /* detect stuff */
+  dir = get_dir_of_binary (argv[0]);
+  base = g_path_get_basename (argv[0]);
 
   /* parse command line options */
-  ctx = g_option_context_new ("gst-run");
+  desc = g_strdup_printf ("wrapper to call versioned %s", base);
+  ctx = g_option_context_new (desc);
+  g_free (desc);
   g_option_context_set_ignore_unknown_options (ctx, TRUE);
   g_option_context_add_main_entries (ctx, wrapper_options, GETTEXT_PACKAGE);
   if (!g_option_context_parse (ctx, &argc, &argv, &err)) {
@@ -303,10 +315,6 @@ main (int argc, char **argv)
   }
   g_option_context_free (ctx);
 
-  /* detect stuff */
-  dir = get_dir_of_binary (argv[0]);
-  base = g_path_get_basename (argv[0]);
-
   /* unmangle libtool if necessary */
   unmangle_libtool (&dir, &base);
 
@@ -326,9 +334,13 @@ main (int argc, char **argv)
     highest = NULL;
 
     /* otherwise, just look up the highest version */
-    g_hash_table_foreach (candidates, (GHFunc) find_highest_version, &highest);
+    if (candidates) {
+      g_hash_table_foreach (candidates, (GHFunc) find_highest_version,
+          &highest);
+    }
+
     if (highest == NULL) {
-      g_print ("ERROR: No version of tool %s not found.\n", base);
+      g_print ("ERROR: No version of tool %s found.\n", base);
       return 1;
     }
     dir = g_hash_table_lookup (candidates, highest);
